//----------------------------------------------------------------------------
/*
 * File: config.c
 * - functions to store and read configuration to/from EEPROM
 * 
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 15 Sep 2009
 * Version 1.3
*/
//----------------------------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/eeprom.h>

#include "config.h"
#include "sd_raw.h"
#include "fat.h"
//----------------------------------------------------------------------------
#define EE_SERIAL_NUMBER_ADR      0
#define EE_SERIAL_NUMBER_SIZE     4
#define EE_DEVICE_TYPE_ADR        (EE_SERIAL_NUMBER_ADR + EE_SERIAL_NUMBER_SIZE)
#define EE_DEVICE_TYPE_SIZE       1
#define EE_PMD32_DRIVE_A_ADR      (EE_DEVICE_TYPE_ADR + EE_DEVICE_TYPE_SIZE)
#define EE_PMD32_DRIVE_A_SIZE     FAT_MAX_PATH
#define EE_PMD32_DRIVE_B_ADR      (EE_PMD32_DRIVE_A_ADR + EE_PMD32_DRIVE_A_SIZE)
#define EE_PMD32_DRIVE_B_SIZE     FAT_MAX_PATH
#define EE_PMD32_DRIVE_C_ADR      (EE_PMD32_DRIVE_B_ADR + EE_PMD32_DRIVE_B_SIZE)
#define EE_PMD32_DRIVE_C_SIZE     FAT_MAX_PATH
#define EE_PMD32_DRIVE_D_ADR      (EE_PMD32_DRIVE_C_ADR + EE_PMD32_DRIVE_C_SIZE)
#define EE_PMD32_DRIVE_D_SIZE     FAT_MAX_PATH
#define EE_PMD32_DRIVE_A_WP_ADR   (EE_PMD32_DRIVE_D_ADR + EE_PMD32_DRIVE_D_SIZE)
#define EE_PMD32_DRIVE_A_WP_SIZE  1
#define EE_PMD32_DRIVE_B_WP_ADR   (EE_PMD32_DRIVE_A_WP_ADR + EE_PMD32_DRIVE_A_WP_SIZE)
#define EE_PMD32_DRIVE_B_WP_SIZE  1
#define EE_PMD32_DRIVE_C_WP_ADR   (EE_PMD32_DRIVE_B_WP_ADR + EE_PMD32_DRIVE_B_WP_SIZE)
#define EE_PMD32_DRIVE_C_WP_SIZE  1
#define EE_PMD32_DRIVE_D_WP_ADR   (EE_PMD32_DRIVE_C_WP_ADR + EE_PMD32_DRIVE_C_WP_SIZE)
#define EE_PMD32_DRIVE_D_WP_SIZE  1

#define EE_LAST_ADDR              (EE_PMD32_DRIVE_D_WP_ADR + EE_PMD32_DRIVE_D_WP_SIZE - 1)
#if EE_LAST_ADDR > E2END
  #error "EEPROM address owerflow!"
#endif
//----------------------------------------------------------------------------
uint8_t config_check_card_sn(void)
{
  struct sd_raw_info disk_info;
  uint32_t sn;

  if (!sd_raw_get_info(&disk_info))
    return 0;

  sn = eeprom_read_dword((uint32_t *)EE_SERIAL_NUMBER_ADR);
  if (sn == disk_info.serial)
    return 1;

  eeprom_write_dword((uint32_t *)EE_SERIAL_NUMBER_ADR, disk_info.serial);

  return 0;
}
//----------------------------------------------------------------------------
uint8_t config_read_device_type(void)
{
  return eeprom_read_byte((uint8_t *)EE_DEVICE_TYPE_ADR);
}
//----------------------------------------------------------------------------
void config_write_device_type(uint8_t dev_type)
{
  eeprom_write_byte((uint8_t *)EE_DEVICE_TYPE_ADR, dev_type);
}
//----------------------------------------------------------------------------
void config_read_pmd32_img_path(uint8_t drv, char *path)
{
  if (drv == 0)
    eeprom_read_block(path, (void *)EE_PMD32_DRIVE_A_ADR, EE_PMD32_DRIVE_A_SIZE);
  else if (drv == 1)
    eeprom_read_block(path, (void *)EE_PMD32_DRIVE_B_ADR, EE_PMD32_DRIVE_B_SIZE);
  else if (drv == 2)
    eeprom_read_block(path, (void *)EE_PMD32_DRIVE_C_ADR, EE_PMD32_DRIVE_C_SIZE);
  else if (drv == 3)
    eeprom_read_block(path, (void *)EE_PMD32_DRIVE_D_ADR, EE_PMD32_DRIVE_D_SIZE);
}
//----------------------------------------------------------------------------
void config_write_pmd32_img_path(uint8_t drv, char *path)
{
  if (drv == 0)
    eeprom_write_block(path, (void *)EE_PMD32_DRIVE_A_ADR, EE_PMD32_DRIVE_A_SIZE);
  else if (drv == 1)
    eeprom_write_block(path, (void *)EE_PMD32_DRIVE_B_ADR, EE_PMD32_DRIVE_B_SIZE);
  else if (drv == 2)
    eeprom_write_block(path, (void *)EE_PMD32_DRIVE_C_ADR, EE_PMD32_DRIVE_C_SIZE);
  else if (drv == 3)
    eeprom_write_block(path, (void *)EE_PMD32_DRIVE_D_ADR, EE_PMD32_DRIVE_D_SIZE);
}
//----------------------------------------------------------------------------
void config_read_pmd32_wp(uint8_t drv, uint8_t *wp)
{
  if (drv == 0)
    *wp = eeprom_read_byte((uint8_t *)EE_PMD32_DRIVE_A_WP_ADR);
  else if (drv == 1)
    *wp = eeprom_read_byte((uint8_t *)EE_PMD32_DRIVE_B_WP_ADR);
  else if (drv == 2)
    *wp = eeprom_read_byte((uint8_t *)EE_PMD32_DRIVE_C_WP_ADR);
  else if (drv == 3)
    *wp = eeprom_read_byte((uint8_t *)EE_PMD32_DRIVE_D_WP_ADR);
}
//----------------------------------------------------------------------------
void config_write_pmd32_wp(uint8_t drv, uint8_t wp)
{
  if (drv == 0)
    eeprom_write_byte((uint8_t *)EE_PMD32_DRIVE_A_WP_ADR, wp);
  else if (drv == 1)
    eeprom_write_byte((uint8_t *)EE_PMD32_DRIVE_B_WP_ADR, wp);
  else if (drv == 2)
    eeprom_write_byte((uint8_t *)EE_PMD32_DRIVE_C_WP_ADR, wp);
  else if (drv == 3)
    eeprom_write_byte((uint8_t *)EE_PMD32_DRIVE_D_WP_ADR, wp);
}
//----------------------------------------------------------------------------
