//-----------------------------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "sd_raw.h"
#include "fat.h"
#include "partition.h"
#include "pgm_strings.h"
#include "timer.h"
#include "btn.h"
#include "lcd.h"
#include "ui.h"
#include "apputil.h"
#include "config.h"
#include "pmd32.h"

#if (1 & DEBUG)
  #include "uart.h"
  #define DEBUG_MAIN  1
#else
  #define DEBUG_MAIN  0
#endif
//-----------------------------------------------------------------------------
#define DEVICE_NONE     -1
#define DEVICE_PMD32    0
#define SET_DATE_TIME   4
//-----------------------------------------------------------------------------
#define ERR_OK          0
#define ERR_ERROR       1
#define ERR_SHOW_MENU   2
#define ERR_DEV_ERROR   3
//-----------------------------------------------------------------------------
int main(void)
{
  char str[LCD_DISPLAY_COLUMNS + 1];
  PGM_P ptxt;
  int8_t mi, err, dev;
  struct partition_struct* part;
  struct fat_fs_struct* fs;

  // main menu
  MENU_ITEM main_menu_items[] = {
    { pstr_main_menu_0, DEVICE_PMD32 },
  };
  MENU main_menu = {
    pstr_main_menu_title,
    sizeof(main_menu_items) / sizeof(MENU_ITEM),
    main_menu_items
  };

  // we will just use ordinary idle mode
  set_sleep_mode(SLEEP_MODE_IDLE);

  // init timer
  timer_init();
  // inti LCD
  lcd_init();
  // init buttons
  btn_init();
#if DEBUG_MAIN
  // init uart
  uart_init();
#endif
  // configure SD card port pins
  sd_raw_configure();

  lcd_backlight_on();

#if DEBUG_MAIN
  uart_puts_p(PSTR("PMD 32-SD Start\n"));
#endif

  // read current device from EEPROM
  dev = (int8_t)config_read_device_type();

  ptxt = 0;
  err = ERR_OK;
  while (1)
  {
    sei();  // enable interrupt

    if (err != ERR_ERROR && err != ERR_DEV_ERROR)
    {
      _delay_ms(200);         // wait some time
      if (btn_test())         // if any button was pressed
      {
        btn_read();
        err = ERR_SHOW_MENU;  // show menu
      }
    }

    mi = -1;
    while (1)
    {
      if (err != ERR_OK && err != ERR_DEV_ERROR)
      {
        if (err == ERR_SHOW_MENU)
        {
          mi = ui_menu(&main_menu, 0);
          if (mi >= 0)
            break;
        }
        else
        {
          if (ptxt)
            lcd_disp_string_at(apputil_center_text_p(str, ptxt, LCD_DISPLAY_COLUMNS), 0, 0);
          lcd_disp_string_at_p(pstr_any_button, 1, 0);
          btn_read();
          err = ERR_SHOW_MENU;
          continue;
        }

        err = ERR_OK;
      }

      if (err == ERR_DEV_ERROR || !sd_raw_init())
      {
        if (!sd_raw_available())
          ptxt = pstr_err_insert_card;
        else
          ptxt = pstr_err_init_card;
        err = ERR_ERROR;
        continue;
      }

      if (dev != DEVICE_NONE)
        break;

      err = ERR_SHOW_MENU;
    }

    if (!sd_raw_init())
    {
      err = ERR_DEV_ERROR;
      continue;
    }

    err = ERR_SHOW_MENU; // show menu after leave device driver main function

    if (mi < 0)
    {
      if (dev != DEVICE_NONE)
        mi = dev;
    }

    // open first partition
    part = partition_open(sd_raw_read, sd_raw_read_interval,
                          sd_raw_write, sd_raw_write_interval, 0);
    if (!part)
    {
      /* If the partition did not open, assume the storage device
       * is a "superfloppy", i.e. has no MBR.
       */
      part = partition_open(sd_raw_read, sd_raw_read_interval,
                            sd_raw_write, sd_raw_write_interval, -1);
      if (!part)
      {
        ptxt = pstr_err_partition;
        err = ERR_ERROR;
        continue;
      }
    }

    // open file system
    fs = fat_open(part);
    if (fs)
    {
      switch (mi)
      {
        case DEVICE_PMD32 :
          dev = DEVICE_PMD32;
          if (config_read_device_type() != DEVICE_PMD32)
            config_write_device_type(DEVICE_PMD32);
          pmd32_main(fs);
          break;

        default :
          ptxt = pstr_err_no_impl;
          err = ERR_ERROR;
          break;
      }
    }
    else
    {
      ptxt = pstr_err_fat;
      err = ERR_ERROR;
      continue;
    }

    // save buffered data block if not saved
    sd_raw_sync();

    // close file system
    if (fs)
    {
      fat_close(fs);
      fs = 0;
    }

    // close partition
    if (part)
    {
      partition_close(part);
      part = 0;
    }

    if (btn_test()) // flush pressed button
      btn_read();
  }

  return 0;
}
//-----------------------------------------------------------------------------
