//-----------------------------------------------------------------------------
/*
 * Copyright (c) 2006-2008 by Roland Riegel <feedback@roland-riegel.de>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
//-----------------------------------------------------------------------------
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/sfr_defs.h>
#include <avr/sleep.h>

#include "uart.h"
//-----------------------------------------------------------------------------
/* some mcus have multiple uarts */
#ifdef UDR0
  #define XUBRRH  UBRR0H
  #define XUBRRL  UBRR0L
  #define XUDR    UDR0

  #define XUCSRA  UCSR0A
  #define XUDRE   UDRE0
  #define XRXC    RXC0

  #define XUCSRB  UCSR0B
  #define XRXEN   RXEN0
  #define XTXEN   TXEN0
  #define XRXCIE  RXCIE0

  #define XUCSRC  UCSR0C
  #define XURSEL
  #define XUCSZ0  UCSZ00
  #define XUCSZ1  UCSZ01

  #define XUCSRC_SELECT 0
#else
  #define XUBRRH  UBRRH
  #define XUBRRL  UBRRL
  #define XUDR    UDR

  #define XUCSRA  UCSRA
  #define XUDRE   UDRE
  #define XRXC    RXC

  #define XUCSRB  UCSRB
  #define XRXEN   RXEN
  #define XTXEN   TXEN
  #define XRXCIE  RXCIE

  #define XUCSRC  UCSRC
  #define XURSEL
  #define XUCSZ0  UCSZ0
  #define XUCSZ1  UCSZ1

  #define XUCSRC_SELECT (1 << URSEL)
#endif

#ifndef USART_RXC_vect
  #if defined(UART0_RX_vect)
    #define USART_RXC_vect UART0_RX_vect
  #elif defined(UART_RX_vect)
    #define USART_RXC_vect UART_RX_vect
  #elif defined(USART0_RX_vect)
    #define USART_RXC_vect USART0_RX_vect
  #elif defined(USART_RX_vect)
    #define USART_RXC_vect USART_RX_vect
  #elif defined(USART0_RXC_vect)
    #define USART_RXC_vect USART0_RXC_vect
  #elif defined(USART_RXC_vect)
    #define USART_RXC_vect USART_RXC_vect
  #else
    #error "Uart receive complete interrupt not defined!"
  #endif
#endif

#define USE_SLEEP   0
#define BAUD        38400UL
#define UBRRVAL     (F_CPU / (BAUD * 16) - 1)
#if (UBRRVAL > 4095)
  #error "Bad value of macro UBRRVAL"
#endif
//-----------------------------------------------------------------------------
void uart_init(void)
{
  /* set baud rate */
  XUBRRH = UBRRVAL >> 8;
  XUBRRL = UBRRVAL & 0xff;
  /* set frame format: 8 bit, no parity, 1 stop bit */
  XUCSRC = XUCSRC_SELECT | (1 << XUCSZ1) | (1 << XUCSZ0);
  /* enable serial receiver and transmitter */
#if !USE_SLEEP
  XUCSRB = (1 << XRXEN) | (1 << XTXEN);
#else
  XUCSRB = (1 << XRXEN) | (1 << XTXEN) | (1 << XRXCIE);
#endif
}
//-----------------------------------------------------------------------------
void uart_putc(uint8_t c)
{
  if (c == '\n')
    uart_putc('\r');

  /* wait until transmit buffer is empty */
  while (!(XUCSRA & (1 << XUDRE)));

  /* send next byte */
  XUDR = c;
}
//-----------------------------------------------------------------------------
void uart_putc_hex(uint8_t b)
{
  /* upper nibble */
  if ((b >> 4) < 0x0a)
    uart_putc((b >> 4) + '0');
  else
    uart_putc((b >> 4) - 0x0a + 'a');

  /* lower nibble */
  if ((b & 0x0f) < 0x0a)
    uart_putc((b & 0x0f) + '0');
  else
    uart_putc((b & 0x0f) - 0x0a + 'a');
}
//-----------------------------------------------------------------------------
void uart_putw_hex(uint16_t w)
{
  uart_putc_hex((uint8_t) (w >> 8));
  uart_putc_hex((uint8_t) (w & 0xff));
}
//-----------------------------------------------------------------------------
void uart_putdw_hex(uint32_t dw)
{
  uart_putw_hex((uint16_t) (dw >> 16));
  uart_putw_hex((uint16_t) (dw & 0xffff));
}
//-----------------------------------------------------------------------------
void uart_putw_dec(uint16_t w)
{
  uint16_t num = 10000;
  uint8_t started = 0;

  while (num > 0)
  {
    uint8_t b = w / num;
    if (b > 0 || started || num == 1)
    {
      uart_putc('0' + b);
      started = 1;
    }
    w -= b * num;

    num /= 10;
  }
}
//-----------------------------------------------------------------------------
void uart_putdw_dec(uint32_t dw)
{
  uint32_t num = 1000000000;
  uint8_t started = 0;

  while (num > 0)
  {
    uint8_t b = dw / num;
    if (b > 0 || started || num == 1)
    {
      uart_putc('0' + b);
      started = 1;
    }
    dw -= b * num;

    num /= 10;
  }
}
//-----------------------------------------------------------------------------
void uart_puts(const char* str)
{
  while (*str)
    uart_putc(*str++);
}
//-----------------------------------------------------------------------------
void uart_puts_p(PGM_P str)
{
  while (1)
  {
    uint8_t b = pgm_read_byte_near(str++);
    if (!b)
      break;

    uart_putc(b);
  }
}
//-----------------------------------------------------------------------------
uint8_t uart_getc(void)
{
  /* wait until receive buffer is full */
#if USE_SLEEP
  uint8_t sreg = SREG;
  sei();

  while (!(XUCSRA & (1 << XRXC)))
  {
    sleep_mode();
  }

  SREG = sreg;
#else
  while (!(XUCSRA & (1 << XRXC)))
    ;
#endif

  uint8_t b = XUDR;
  if (b == '\r')
    b = '\n';

  return b;
}
//-----------------------------------------------------------------------------
EMPTY_INTERRUPT(USART_RXC_vect)
//-----------------------------------------------------------------------------
